@@ -1,4 +1,3 @@
-
 /*
  * Copyright (c) 1997-2003 by The XFree86 Project, Inc.
  *
@@ -40,58 +39,55 @@
 #include "xf86.h"
 #include "xf86Priv.h"
 #ifdef DPMSExtension
-#define DPMS_SERVER
-#include <X11/extensions/dpms.h>
+#include <X11/extensions/dpmsconst.h>
 #include "dpmsproc.h"
 #endif
+#include "xf86VGAarbiter.h"
 
 
 #ifdef DPMSExtension
-static int DPMSGeneration = 0;
-static int DPMSIndex = -1;
+static int DPMSKeyIndex;
+static DevPrivateKey DPMSKey;
 static Bool DPMSClose(int i, ScreenPtr pScreen);
 static int DPMSCount = 0;
 #endif
 
 
-_X_EXPORT Bool
+Bool
 xf86DPMSInit(ScreenPtr pScreen, DPMSSetProcPtr set, int flags)
 {
 #ifdef DPMSExtension
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     DPMSPtr pDPMS;
     pointer DPMSOpt;
+    MessageType enabled_from;
 
-    if (serverGeneration != DPMSGeneration) {
-	if ((DPMSIndex = AllocateScreenPrivateIndex()) < 0)
-	    return FALSE;
-	DPMSGeneration = serverGeneration;
-    }
+    DPMSKey = &DPMSKeyIndex;
 
-    if (DPMSDisabledSwitch)
-	DPMSEnabled = FALSE;
-    if (!(pScreen->devPrivates[DPMSIndex].ptr = xcalloc(sizeof(DPMSRec), 1)))
+    if (!dixSetPrivate(&pScreen->devPrivates, DPMSKey,
+		       xcalloc(sizeof(DPMSRec), 1)))
 	return FALSE;
 
-    pDPMS = (DPMSPtr)pScreen->devPrivates[DPMSIndex].ptr;
+    pDPMS = dixLookupPrivate(&pScreen->devPrivates, DPMSKey);
     pScrn->DPMSSet = set;
     pDPMS->Flags = flags;
     DPMSOpt = xf86FindOption(pScrn->options, "dpms");
-    if (DPMSOpt) {
-	if ((pDPMS->Enabled
-	    = xf86SetBoolOption(pScrn->options, "dpms", FALSE))
-	    && !DPMSDisabledSwitch)
-	    DPMSEnabled = TRUE;
-	xf86MarkOptionUsed(DPMSOpt);
-	xf86DrvMsg(pScreen->myNum, X_CONFIG, "DPMS enabled\n");
-    } else if (DPMSEnabledSwitch) {
-	if (!DPMSDisabledSwitch)
-	    DPMSEnabled = TRUE;
-	pDPMS->Enabled = TRUE;
-    }  
-    else {
-	pDPMS->Enabled = defaultDPMSEnabled;
+    if (DPMSDisabledSwitch) {
+	enabled_from = X_CMDLINE;
+	DPMSEnabled = FALSE;
     }
+    else if (DPMSOpt) {
+	enabled_from = X_CONFIG;
+	DPMSEnabled = xf86CheckBoolOption(pScrn->options, "dpms", FALSE);
+	xf86MarkOptionUsed(DPMSOpt);
+    }
+    else {
+	enabled_from = X_DEFAULT;
+	DPMSEnabled = TRUE;
+    }
+    if (DPMSEnabled)
+	xf86DrvMsg(pScreen->myNum, enabled_from, "DPMS enabled\n");
+    pDPMS->Enabled = DPMSEnabled;
     pDPMS->CloseScreen = pScreen->CloseScreen;
     pScreen->CloseScreen = DPMSClose;
     DPMSCount++;
@@ -110,10 +106,10 @@ DPMSClose(int i, ScreenPtr pScreen)
     DPMSPtr pDPMS;
 
     /* This shouldn't happen */
-    if (DPMSIndex < 0)
+    if (DPMSKey == NULL)
 	return FALSE;
 
-    pDPMS = (DPMSPtr)pScreen->devPrivates[DPMSIndex].ptr;
+    pDPMS = dixLookupPrivate(&pScreen->devPrivates, DPMSKey);
 
     /* This shouldn't happen */
     if (!pDPMS)
@@ -131,10 +127,10 @@ DPMSClose(int i, ScreenPtr pScreen)
  	xf86Screens[i]->DPMSSet(xf86Screens[i],DPMSModeOn,0);
     }
     
-    xfree((pointer)pDPMS);
-    pScreen->devPrivates[DPMSIndex].ptr = NULL;
+    xfree(pDPMS);
+    dixSetPrivate(&pScreen->devPrivates, DPMSKey, NULL);
     if (--DPMSCount == 0)
-	DPMSIndex = -1;
+	DPMSKey = NULL;
     return pScreen->CloseScreen(i, pScreen);
 }
 
@@ -144,30 +140,35 @@ DPMSClose(int i, ScreenPtr pScreen)
  *	Device dependent DPMS mode setting hook.  This is called whenever
  *	the DPMS mode is to be changed.
  */
-_X_EXPORT void
-DPMSSet(int level)
+int
+DPMSSet(ClientPtr client, int level)
 {
-    int i;
+    int rc, i;
     DPMSPtr pDPMS;
     ScrnInfoPtr pScrn;
 
     DPMSPowerLevel = level;
 
-    if (DPMSIndex < 0)
-	return;
+    if (DPMSKey == NULL)
+	return Success;
 
-    if (level != DPMSModeOn)
-	SaveScreens(SCREEN_SAVER_FORCER, ScreenSaverActive);
+    if (level != DPMSModeOn) {
+	rc = dixSaveScreens(client, SCREEN_SAVER_FORCER, ScreenSaverActive);
+	if (rc != Success)
+	    return rc;
+    }
 
     /* For each screen, set the DPMS level */
     for (i = 0; i < xf86NumScreens; i++) {
     	pScrn = xf86Screens[i];
-	pDPMS = (DPMSPtr)screenInfo.screens[i]->devPrivates[DPMSIndex].ptr;
+	pDPMS = dixLookupPrivate(&screenInfo.screens[i]->devPrivates, DPMSKey);
 	if (pDPMS && pScrn->DPMSSet && pDPMS->Enabled && pScrn->vtSema) { 
-	    xf86EnableAccess(pScrn);
+	    xf86VGAarbiterLock(pScrn);
 	    pScrn->DPMSSet(pScrn, level, 0);
+	    xf86VGAarbiterUnlock(pScrn);
 	}
     }
+    return Success;
 }
 
 
@@ -175,46 +176,25 @@ DPMSSet(int level)
  * DPMSSupported --
  *	Return TRUE if any screen supports DPMS.
  */
-_X_EXPORT Bool
+Bool
 DPMSSupported(void)
 {
     int i;
     DPMSPtr pDPMS;
     ScrnInfoPtr pScrn;
 
-    if (DPMSIndex < 0) {
+    if (DPMSKey == NULL) {
 	return FALSE;
     }
 
     /* For each screen, check if DPMS is supported */
     for (i = 0; i < xf86NumScreens; i++) {
     	pScrn = xf86Screens[i];
-	pDPMS = (DPMSPtr)screenInfo.screens[i]->devPrivates[DPMSIndex].ptr;
+	pDPMS = dixLookupPrivate(&screenInfo.screens[i]->devPrivates, DPMSKey);
 	if (pDPMS && pScrn->DPMSSet)
 	    return TRUE;
     }
     return FALSE;
 }
 
-
-/*
- * DPMSGet --
- *	Device dependent DPMS mode getting hook.  This returns the current
- *	DPMS mode, or -1 if DPMS is not supported.
- *
- *	This should hook in to the appropriate driver-level function, which
- *	will be added to the ScrnInfoRec.
- *
- *	NOTES:
- *	 1. the calling interface should be changed to specify which
- *	    screen to check.
- *	 2. It isn't clear that this function is ever used or what it should
- *	    return.
- */
-_X_EXPORT int
-DPMSGet(int *level)
-{
-    return DPMSPowerLevel;
-}
-
 #endif /* DPMSExtension */
