@@ -113,7 +113,7 @@ ProcessVelocityConfiguration(DeviceIntPtr pDev, char* devname, pointer list,
         return;
 
     /* common settings (available via device properties) */
-    tempf = xf86SetRealOption(list, "ConstantDeceleration", 1.0);
+    tempf = xf86SetIntOption(list, "ConstantDeceleration", 1);
     if(tempf > 1.0){
         xf86Msg(X_CONFIG, "%s: (accel) constant deceleration by %.1f\n",
                 devname, tempf);
@@ -122,7 +122,7 @@ ProcessVelocityConfiguration(DeviceIntPtr pDev, char* devname, pointer list,
                                PropModeReplace, 1, &tempf, FALSE);
     }
 
-    tempf = xf86SetRealOption(list, "AdaptiveDeceleration", 1.0);
+    tempf = xf86SetIntOption(list, "AdaptiveDeceleration", 1);
     if(tempf > 1.0){
         xf86Msg(X_CONFIG, "%s: (accel) adaptive deceleration by %.1f\n",
                 devname, tempf);
@@ -726,14 +726,11 @@ xf86PostMotionEventP(DeviceIntPtr	device,
                     int			*valuators)
 {
     int i = 0, nevents = 0;
+    int dx = 0, dy = 0;
     Bool drag = xf86SendDragEvents(device);
     DeviceEvent *event;
-    int flags = 0;
-
-#if XFreeXDGA
     int index;
-    int dx = 0, dy = 0;
-#endif
+    int flags = 0;
 
     XI_VERIFY_VALUATORS(num_valuators);
 
@@ -859,11 +856,8 @@ xf86PostButtonEventP(DeviceIntPtr	device,
                      int		*valuators)
 {
     int i = 0, nevents = 0;
-    int flags = 0;
-
-#if XFreeXDGA
     int index;
-#endif
+    int flags = 0;
 
     XI_VERIFY_VALUATORS(num_valuators);
 
